/* $Id: messagenr.h,v 1.2 1998/05/10 23:40:13 luethje Exp $
 *
 * kisdnog for ISDN accounting for isdn4linux. (Report-module)
 *
 * Copyright 1996, 1997 by Stefan Luethje (luethje@sl-gw.lake.de)
 *                         Claudia Weber  (weber@sl-gw.lake.de)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Log: messagenr.h,v $
 * Revision 1.2  1998/05/10 23:40:13  luethje
 * some changes
 *
 */

#define KI_OUT_OF_MEMORY	   1
#define	KI_NO_FILE				   2
#define	KI_NO_DIR					   3
#define KI_NO_HOST				 101
#define KI_NO_CONNECT			 102
#define KI_NOBODY					 103
#define KI_CONN_BROKEN		 201
#define KI_ACC_DENIED			 202
#define KI_UNKNOWN_MSG		 203
#define KI_WRONG_RESULT		 204
#define KI_UNKNOWN_MSG2		 205
#define KI_DIFF_VERSION		 206
#define KI_UNKNOWN_MSG3		 301
#define KI_INVALID_PORT		1001
#define KI_INVALID_ELEMS	1101
#define KI_INVALID_LINES	1201
